use strict;
use Test::More tests => 1;

BEGIN {
    use_ok( 'NetworkInfo::Discovery::Rendezvous' );
}

diag( "Testing NetworkInfo::Discovery::Rendezvous $NetworkInfo::Discovery::Rendezvous::VERSION" );

